@@ -8,7 +8,6 @@ import contextlib
 import shutil
 import time
 import tempfile
-import tarfile
 
 # the code in this file was originally inspired by similar code for R
 # https://github.com/beanumber/abdwr3edata/blob/main/R/parse_retrosheet_pbp.R
@@ -43,6 +42,7 @@ def prepend_path(dir):
 # names originally from:
 # https://github.com/maxtoki/baseball_R/blob/master/data/game_log_header.csv
 # changed HomeRunsScore to HomeRunsScored which seems to have been a typo
+# LosingPitcherNAme -> LosingPitcherName (also typo?)
 gamelogs_header = [
     "Date",
     "DoubleHeader",
@@ -140,7 +140,7 @@ gamelogs_header = [
     "WinningPitcherID",
     "WinningPitcherName",
     "LosingPitcherID",
-    "LosingPitcherNAme",
+    "LosingPitcherName",  # fixed typo (LosingPitcherNAme -> LosingPitcherName)
     "SavingPitcherID",
     "SavingPitcherName",
     "GameWinningRBIID",
@@ -446,15 +446,8 @@ def download_gamelogs(start_year, end_year=None, data_dir="retrosheet"):
             zipped_file = os.path.join(temp_dir, f"gl{year}.zip")
             gamelogs_txt = os.path.join(temp_dir, f"gl{year}.txt")
 
-            # obtain zipped gamelogs from retrosheet website
-            r = requests.get(url, timeout=30)
-            r.raise_for_status()
-            with open(zipped_file, "wb") as f:
-                f.write(r.content)
-
-            # decompress downloaded gamelogs
-            with zipfile.ZipFile(zipped_file, "r") as zip_ref:
-                zip_ref.extractall(temp_dir)
+            # obtain zipped gamelogs from retrosheet website and decompress
+            _download_and_unzip(url, zipped_file, temp_dir)
 
             # parse the gamelogs
             df = pd.read_csv(
@@ -481,48 +474,46 @@ def load_gamelogs(year, data_dir="retrosheet", force_download=False):
     return pd.read_parquet(gamelogs_parquet, engine="pyarrow")
 
 
-def download_chadwick(install_dir="chadwick"):
+def download_chadwick_windows(install_dir="chadwick", approve=False):
+    if not approve:
+        raise RuntimeError(
+            "You must set approve=True to acknowledge that you are downloading "
+            "and installing executable files that will be run on your system. "
+            "These executables are from the Chadwick Baseball Bureau project: "
+            "https://github.com/chadwickbureau/chadwick - please review the "
+            "project and verify it is safe before proceeding."
+        )
+
     install_dir = os.path.join(os.getcwd(), install_dir)
     os.makedirs(install_dir, exist_ok=True)
-    if os.name == "nt":
-        url = "https://github.com/chadwickbureau/chadwick/releases/download/v0.10.0/chadwick-0.10.0-win.zip"
-        filename = "chadwick-0.10.0-win.zip"
-        print(f"Downloading Chadwick tools to {install_dir}")
-    else:
-        url = "https://github.com/chadwickbureau/chadwick/releases/download/v0.10.0/chadwick-0.10.0.tar.gz"
-        filename = "chadwick-0.10.0.tar.gz"
-        print(f"This function currently only supports Windows.")
-        return None
+
+    url = "https://github.com/chadwickbureau/chadwick/releases/download/v0.10.0/chadwick-0.10.0-win.zip"
+    filename = "chadwick-0.10.0-win.zip"
+    print(f"Downloading Chadwick tools for Windows to {install_dir}")
+
     with tempfile.TemporaryDirectory() as temp_dir:
         archive_file = os.path.join(temp_dir, filename)
-        print(f"Downloading {url}")
-        r = requests.get(url, timeout=30)
-        r.raise_for_status()
-        with open(archive_file, "wb") as f:
-            f.write(r.content)
-        print(f"Extracting to {temp_dir}")
-        if filename.endswith(".zip"):
-            with zipfile.ZipFile(archive_file, "r") as zip_ref:
-                zip_ref.extractall(temp_dir)
-        elif filename.endswith(".tar.gz"):
-            with tarfile.open(archive_file, "r:gz") as tar_ref:
-                tar_ref.extractall(temp_dir, filter="data")
-        extracted_dirs = [
-            d
-            for d in os.listdir(temp_dir)
-            if os.path.isdir(os.path.join(temp_dir, d)) and not d.startswith(".")
-        ]
-        if extracted_dirs:
-            chadwick_dir = os.path.join(temp_dir, extracted_dirs[0])
+
+        # download and extract using the helper function with retry logic
+        _download_and_unzip(url, archive_file, temp_dir)
+
+        # find the Chadwick executables
+        exe_files = [f for f in os.listdir(temp_dir) if f.endswith(".exe")]
+        if exe_files:
+            chadwick_dir = temp_dir
         else:
-            if os.name == "nt":
-                exe_files = [f for f in os.listdir(temp_dir) if f.endswith(".exe")]
-            else:
-                exe_files = None
-            if exe_files:
-                chadwick_dir = temp_dir
+            # look for extracted directory
+            extracted_dirs = [
+                d
+                for d in os.listdir(temp_dir)
+                if os.path.isdir(os.path.join(temp_dir, d)) and not d.startswith(".")
+            ]
+            if extracted_dirs:
+                chadwick_dir = os.path.join(temp_dir, extracted_dirs[0])
             else:
                 raise RuntimeError("No Chadwick tools found in extracted archive")
+
+        # copy files to install directory
         for item in os.listdir(chadwick_dir):
             if item.endswith((".zip", ".tar.gz")) or item.startswith("."):
                 continue
@@ -532,7 +523,150 @@ def download_chadwick(install_dir="chadwick"):
                 shutil.copytree(src, dst, dirs_exist_ok=True)
             else:
                 shutil.copy2(src, dst)
+
     print(f"Chadwick tools installed to: {install_dir}")
-    if os.name == "nt":
-        print(f"You can now use cwevent_dir='{install_dir}' in download_events().")
+    print(f"You can now use cwevent_dir='{install_dir}' in download_events().")
     return install_dir
+
+
+def download_chadwick(install_dir="chadwick", approve=False):
+    if os.name == "nt":
+        return download_chadwick_windows(install_dir, approve=approve)
+    else:
+        print("download_chadwick currently only supports Windows.")
+        print("For other platforms, please install Chadwick tools manually.")
+        print(
+            "Installation instructions can be found at: https://github.com/chadwickbureau/chadwick/blob/master/INSTALL"
+        )
+        return None
+
+
+def add_states(df):
+    return df.assign(
+        # create bases string (000: none on, 100: runner on first, etc)
+        BASES=lambda x: (
+            ((x["BASE1_RUN_ID"] != "") & (x["BASE1_RUN_ID"].notna())).astype(int).astype(str)
+            + ((x["BASE2_RUN_ID"] != "") & (x["BASE2_RUN_ID"].notna())).astype(int).astype(str)
+            + ((x["BASE3_RUN_ID"] != "") & (x["BASE3_RUN_ID"].notna())).astype(int).astype(str)
+        ),
+        # create state (bases + outs)
+        STATE=lambda x: x["BASES"] + " " + x["OUTS_CT"].astype(str),
+        # determine if runners advance to each base
+        IS_RUNNER1=lambda x: ((x["RUN1_DEST_ID"] == 1) | (x["BAT_DEST_ID"] == 1)).astype(int),
+        IS_RUNNER2=lambda x: (
+            (x["RUN1_DEST_ID"] == 2) | (x["RUN2_DEST_ID"] == 2) | (x["BAT_DEST_ID"] == 2)
+        ).astype(int),
+        IS_RUNNER3=lambda x: (
+            (x["RUN1_DEST_ID"] == 3)
+            | (x["RUN2_DEST_ID"] == 3)
+            | (x["RUN3_DEST_ID"] == 3)
+            | (x["BAT_DEST_ID"] == 3)
+        ).astype(int),
+        # calculate new outs, bases, and state
+        NEW_OUTS=lambda x: x["OUTS_CT"] + x["EVENT_OUTS_CT"],
+        NEW_BASES=lambda x: (
+            x["IS_RUNNER1"].astype(str) + x["IS_RUNNER2"].astype(str) + x["IS_RUNNER3"].astype(str)
+        ),
+        NEW_STATE=lambda x: x["NEW_BASES"] + " " + x["NEW_OUTS"].astype(str),
+        # calculate runs scored
+        RUNS_SCORED=lambda x: (
+            (x["BAT_DEST_ID"] > 3).astype(int)
+            + (x["RUN1_DEST_ID"] > 3).astype(int)
+            + (x["RUN2_DEST_ID"] > 3).astype(int)
+            + (x["RUN3_DEST_ID"] > 3).astype(int)
+        ),
+        # get current total score
+        RUNS_BEFORE=lambda x: x["AWAY_SCORE_CT"] + x["HOME_SCORE_CT"],
+        # create half inning identifier
+        HALF_INNING=lambda x: (
+            x["GAME_ID"].astype(str) + x["INN_CT"].astype(str) + x["BAT_HOME_ID"].astype(str)
+        ),
+    )
+
+
+def add_counts(df):
+
+    # copy input df
+    df = df.copy()
+
+    # define ball and strike patterns
+    b = "[BIPV]"  # balls
+    s = "[CFKLMOQRST]"  # strikes
+
+    # clean pitch sequence by removing non-ball/strike characters
+    df["PSEQ"] = df["PITCH_SEQ_TX"].str.replace(r"[.>123+*N]", "", regex=True)
+
+    # Count indicators (True if the count was reached during the at-bat)
+    df["C00"] = True  # Every at-bat starts at 0-0
+
+    # 1-0, 0-1 counts
+    df["C10"] = df["PSEQ"].str.match(f"^{b}", na=False)
+    df["C01"] = df["PSEQ"].str.match(f"^{s}", na=False)
+
+    # 2-0, 3-0 counts
+    df["C20"] = df["PSEQ"].str.match(f"^{b}{{2}}", na=False)
+    df["C30"] = df["PSEQ"].str.match(f"^{b}{{3}}", na=False)
+
+    # 0-2 count
+    df["C02"] = df["PSEQ"].str.match(f"^{s}{{2}}", na=False)
+
+    # 1-1 count
+    df["C11"] = df["PSEQ"].str.match(f"^{s}{b}|^{b}{s}", na=False)
+
+    # 2-1 count
+    df["C21"] = df["PSEQ"].str.match(f"^{s}{b}{b}|^{b}{s}{b}|^{b}{b}{s}", na=False)
+
+    # 3-1 count
+    df["C31"] = df["PSEQ"].str.match(
+        f"^{s}{b}{b}{b}|^{b}{s}{b}{b}|^{b}{b}{s}{b}|^{b}{b}{b}{s}", na=False
+    )
+
+    # 1-2 count
+    df["C12"] = df["PSEQ"].str.match(f"^{b}{s}{s}|^{s}{b}{s}|^{s}{s}[FR]*{b}", na=False)
+
+    # 2-2 count
+    df["C22"] = df["PSEQ"].str.match(
+        f"^{b}{b}{s}{s}|^{b}{s}{b}{s}|^{b}{s}{s}[FR]*{b}|^{s}{b}{b}{s}|^{s}{b}{s}[FR]*{b}|^{s}{s}[FR]*{b}[FR]*{b}",
+        na=False,
+    )
+
+    # 3-2 count (full count)
+    has_3_balls = df["PSEQ"].str.match(f"^{s}*{b}{s}*{b}{s}*{b}", na=False)
+    has_2_strikes = df["PSEQ"].str.match(f"^{b}*{s}{b}*{s}", na=False)
+    df["C32"] = has_3_balls & has_2_strikes
+
+    # return df with added counts
+    return df
+
+
+def make_half_innings(df):
+
+    # copy input df
+    df = df.copy()
+
+    # make half innings
+    half_innings = (
+        df.groupby("HALF_INNING")
+        .agg(
+            {
+                "EVENT_OUTS_CT": "sum",
+                "RUNS_SCORED": "sum",
+                "RUNS_BEFORE": "first",
+            }
+        )
+        .rename(
+            columns={
+                "EVENT_OUTS_CT": "OUTS_INNING",
+                "RUNS_SCORED": "RUNS_INNING",
+                "RUNS_BEFORE": "RUNS_START",
+            }
+        )
+        .assign(MAX_RUNS=lambda df: df["RUNS_INNING"] + df["RUNS_START"])
+        .reset_index()
+    )
+
+    # return half innings
+    return half_innings
+
+
+# TODO: fun that reduces columns to those needed for state based analysis
