@@ -24,7 +24,7 @@ def download_seasons(start_year, end_year=None, force_download=False, data_dir="
 
     # process all years requested
     for year in range(start_year, end_year + 1):
-        parquet_path = os.path.join(data_dir, f"sc-{year}.parquet")
+        parquet_path = os.path.join(data_dir, f"statcast-{year}.parquet")
         if os.path.exists(parquet_path) and not force_download:
             print(f"Data for {year} already exists, skipping.")
         else:
@@ -37,16 +37,11 @@ def download_seasons(start_year, end_year=None, force_download=False, data_dir="
 
 
 def load_season(year, force_download=False, regular_season=False, data_dir="statcast"):
-    parquet_path = os.path.join(data_dir, f"sc-{year}.parquet")
-    parquet_path_in_data = None
-    if data_dir == "statcast":
-        parquet_path_in_data = os.path.join("data", data_dir, f"sc-{year}.parquet")
+    parquet_path = os.path.join(data_dir, f"statcast-{year}.parquet")
     if force_download:
         download_seasons(year, year, force_download=force_download, data_dir=data_dir)
     if os.path.exists(parquet_path):
         df = pd.read_parquet(parquet_path, engine="pyarrow")
-    elif parquet_path_in_data and os.path.exists(parquet_path_in_data):
-        df = pd.read_parquet(parquet_path_in_data, engine="pyarrow")
     else:
         download_seasons(year, year, data_dir=data_dir)
         df = pd.read_parquet(parquet_path, engine="pyarrow")
@@ -287,8 +282,10 @@ def reorder_columns(df, keep_all=False):
     return df
 
 
-# NOTE: currently defaulting to ascending=False to match pybaseball
-def arrange_rows(df, keep_index=False, ascending=False):
+# TODO: note that game *time* is not considered
+# TODO: add uid for unique at-bat and pitch?
+# TODO: probably make a copy somewhere
+def order_pitches(df, keep_index=False, ascending=True, add_uid=False):
     sort_columns = ["game_date", "game_pk", "at_bat_number", "pitch_number"]
     missing = [col for col in sort_columns if col not in df.columns]
     if missing:
@@ -305,6 +302,7 @@ def filter_regular_season(df):
     return df[df["game_type"] == "R"].copy()
 
 
+# TODO: don't do in-place, make a copy somewhere? (edit in book)
 def add_transformed_hit_locations(df):
     a_x, b_x = (-301.777, 2.41036)
     a_y, b_y = (490.267, -2.41036)
@@ -312,9 +310,10 @@ def add_transformed_hit_locations(df):
     df["hit_y"] = a_y + b_y * df["hc_y"]
 
 
+# TODO: don't do in-place, make a copy somewhere? (edit in book)
 def add_names(df, remove_player=True):
-    df["batter_name"] = get_name_from_id(df["batter"])
-    df["pitcher_name"] = get_name_from_id(df["pitcher"])
+    df["batter_name"] = lookup_name(df["batter"])
+    df["pitcher_name"] = lookup_name(df["pitcher"])
     if remove_player:
         df.drop("player_name", axis=1, inplace=True)
 
@@ -380,12 +379,12 @@ def _get_chadwick_people():
     return _chadwick_people
 
 
-def get_name_from_id(mlbam_id):
+def lookup_name(mlbam_id):
 
     # get people df
     people = _get_chadwick_people()
 
-    # build name from id
+    # build name from components
     def name_for_row(row):
         first = row.get("name_first", "")
         if pd.isna(first):
@@ -396,32 +395,91 @@ def get_name_from_id(mlbam_id):
         suffix = row.get("name_suffix", "")
         if pd.isna(suffix):
             suffix = ""
-        return f"{first} {last} {suffix}".strip()
+        parts = [part.strip() for part in [first, last, suffix] if part and str(part).strip()]
+        return " ".join(parts) if parts else ""
 
-    # build fast look mapping
-    if not hasattr(get_name_from_id, "_id_to_name"):
-        get_name_from_id._id_to_name = {
-            int(row["key_mlbam"]): name_for_row(row)
-            for _, row in people.iterrows()
-            if pd.notna(row["key_mlbam"])
-        }
-    id_to_name = get_name_from_id._id_to_name
+    # build fast lookup mapping
+    if not hasattr(lookup_name, "_id_to_name"):
+        valid_people = people[people["key_mlbam"].notna()].copy()
+        if len(valid_people) > 0:
+            # vectorized name construction
+            names = valid_people.apply(name_for_row, axis=1)
+            ids = valid_people["key_mlbam"].astype(int)
+            lookup_name._id_to_name = dict(zip(ids, names))
+        else:
+            lookup_name._id_to_name = {}
+    id_to_name = lookup_name._id_to_name
 
     # handle different input types
     if isinstance(mlbam_id, int):
         return id_to_name.get(mlbam_id, None)
     elif isinstance(mlbam_id, list):
-        return [id_to_name.get(mid, None) for mid in mlbam_id]
-    elif isinstance(mlbam_id, pd.Series) and pd.api.types.is_integer_dtype(mlbam_id):
-        return [id_to_name.get(mid, None) for mid in mlbam_id.to_list()]
+        if not mlbam_id:  # empty list
+            return []
+        # validate all items are integers
+        try:
+            return [id_to_name.get(int(mid), None) for mid in mlbam_id]
+        except (ValueError, TypeError) as e:
+            raise ValueError(f"All items in list must be convertible to integers: {e}")
+    elif isinstance(mlbam_id, pd.Series):
+        if len(mlbam_id) == 0:  # empty series
+            return []
+        # check if series contains integer-like data
+        if not (
+            pd.api.types.is_integer_dtype(mlbam_id)
+            or pd.api.types.is_extension_array_dtype(mlbam_id)
+        ):
+            raise TypeError("pandas Series must have integer dtype")
+
+        # convert each value, handling NaN appropriately
+        result = []
+        for value in mlbam_id:
+            if pd.isna(value):
+                result.append(None)
+            else:
+                try:
+                    result.append(id_to_name.get(int(value), None))
+                except (ValueError, TypeError):
+                    raise ValueError(f"Cannot convert Series value {value} to integer")
+        return result
     else:
-        raise TypeError("mlbam_id must be an integer, a list of integers, or an integer Series.")
+        raise TypeError(
+            "mlbam_id must be an integer, a list of integers, or an integer pandas Series."
+        )
 
 
-# TODO*: different versions: search within full name versus by element?
-def get_id_from_name(name_last, name_first):
+def clear_name_cache():
+    """
+    Clear the cached player name lookup dictionary.
+
+    This can be useful if the underlying Chadwick data has been updated
+    or to free up memory in long-running applications.
+    """
+    if hasattr(lookup_name, "_id_to_name"):
+        delattr(lookup_name, "_id_to_name")
+
+
+def lookup_id(name_last, name_first, last_only=False, fuzzy=False):
     people = _get_chadwick_people()
-    mask = (people["name_first"].fillna("").str.lower() == str(name_first).strip().lower()) & (
-        people["name_last"].fillna("").str.lower() == str(name_last).strip().lower()
-    )
-    return people.loc[mask]
+    if not name_last:
+        raise ValueError("name_last cannot be empty")
+    if not last_only and not name_first:
+        raise ValueError("name_first cannot be empty when last_only=False")
+    name_last_people = people["name_last"].fillna("").str.lower()
+    name_first_people = people["name_first"].fillna("").str.lower()
+    name_first = str(name_first).strip().lower() if name_first else ""
+    name_last = str(name_last).strip().lower()
+    last_mask = name_last_people == name_last
+    if last_only:
+        if fuzzy and not last_mask.any():
+            last_fuzzy = name_last_people.str.contains(name_last, na=False, regex=False)
+            return people.loc[last_fuzzy]
+        return people.loc[last_mask]
+    first_mask = name_first_people == name_first
+    full_mask = last_mask & first_mask
+    if fuzzy and not full_mask.any():
+        last_fuzzy = name_last_people.str.contains(name_last, na=False, regex=False)
+        first_fuzzy = name_first_people.str.contains(name_first, na=False, regex=False)
+        fuzzy_mask = last_fuzzy & first_fuzzy
+        return people.loc[fuzzy_mask]
+    return people.loc[full_mask]
