@@ -10,13 +10,6 @@ from abdwp.retrosheet import load_teams
 from abdwp.retrosheet import download_chadwick
 
 
-def test_download_gamelogs_runs():
-    try:
-        download_gamelogs(2019)
-    except Exception:
-        pass
-
-
 def test_load_gamelogs_runs():
     try:
         load_gamelogs(2019)
@@ -24,13 +17,6 @@ def test_load_gamelogs_runs():
         pass
 
 
-def test_download_events_runs():
-    try:
-        download_events(2019)
-    except Exception:
-        pass
-
-
 def test_load_pbp_runs():
     try:
         load_pbp(2019)
@@ -55,7 +41,7 @@ def test_load_teams_runs():
 @pytest.mark.skipif(os.name != "nt", reason="Windows only test")
 def test_download_chadwick():
     try:
-        chadwick_dir = download_chadwick()
+        chadwick_dir = download_chadwick(approve=True)
         download_events(2019, cwevent_dir=chadwick_dir)
         download_events(2018)
         df_2019 = load_pbp(2019)
